/**************************************************************************************************
 *
 * FileName:        main_lcd_graphics_demo.c
 *
 * Overview:        Test and demo utility for ST7920 LCD graphics functions
 * `````````        and 4-wire 4x4 button keypad.
 *
 * Processor:       PIC32MX460 or PIC32MX340
 *
 * H/W Platform:    Olimex PIC32-MX460 development board, or
 * `````````````    Olimex PIC32-MX340 prototyping board, or
 *                  compatible target platform.
 *
 * Compiler:        Microchip MPLAB XC32 v1.33 or later (free version).
 *                  The project contains no Microchip "PLIB" dependencies.
 *
 * Author:          M.J.Bauer, 2015   [www.mjbauer.biz]
 *
 * Reference:       http:/www.mjbauer.biz/mjb_resources.htm
 *
 * History:         See ../doc/RevisionHistory_xxxx.txt
 *
 * ================================================================================================
 */
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "system_def.h"
#include "main_lcd_graphics_demo.h"


uint8 g_FW_version[4];    // firmware version # (major, minor, build, 0)

uint8  test_image[] =     // bitmap data, test using w = 16..24 (typ 20), h <= 20
{
        0xF0, 0x00, 0xF0,
        0xF0, 0x00, 0xF0,
        0xF0, 0x00, 0xF0,
        0xF0, 0x00, 0xF0,
        0x0F, 0x0F, 0x00,
        0x0F, 0x0F, 0x00,
        0x0F, 0x0F, 0x00,
        0x0F, 0x0F, 0x00,
        0x00, 0xF0, 0x00,
        0x00, 0xF0, 0x00,
        0x00, 0xF0, 0x00,
        0x00, 0xF0, 0x00,
        0x0F, 0x0F, 0x00,
        0x0F, 0x0F, 0x00,
        0x0F, 0x0F, 0x00,
        0x0F, 0x0F, 0x00,
        0xF0, 0x00, 0xF0,
        0xF0, 0x00, 0xF0,
        0xF0, 0x00, 0xF0,
        0xF0, 0x00, 0xF0
};


// A user-settable parameter table is not required in this application,
// but we need to define a dummy table so as to include the console CLI module.
//
const  UserSettableParameter_t  UserParam[] =    // For use with "set" command
    {
//      { "volume",    'i',  &g_AudioVolume, 0, 100 },
        //-----------------------------------------------------
        { "$",         'r',  NULL, 0.0, 0.0 }  // Last entry!
    };

//=================================================================================================

static void  Init_Application(void)
{
    unsigned int time_ms;
    int  count;

    g_FW_version[0] = BUILD_VER_MAJOR;
    g_FW_version[1] = BUILD_VER_MINOR;
    g_FW_version[2] = BUILD_VER_DEBUG;

    // Initialize I/O, periph's. (in low-level.c)
    Init_MCU_IO_ports();

    UART2_init(57600);   // for console using UART2
    Init_I2C2();         // for RTCC, ext I/O, etc

    time_ms = millisecTimer();
    for (count = 0;  count < 1000;  count++)
    {
        Delay10us(10);  // delay 100us  (in TimeDelay.c)
    }
    time_ms = millisecTimer() - time_ms;  // elapsed time (ms)

    if (time_ms < 90 || time_ms > 110)  // should be 100ms (+/-10%)
    {
        putstr("\n! Timing error in Delay10us(): ");
        putDecimal(((((int)time_ms - 100) * 100)/100), 1);
        putstr("% \n");
    }

#if defined (RTCC_TYPE_ISL1208) || defined (RTCC_TYPE_MCP79410)
    RTCC_Synchronize();  // Set time & date from hardware RTCC
#endif

    KeypadTaskEnable(1);   // Enable keypad driver ISR
}


void  DisplayHomeMessageScreen()
{
    LCD_ClearScreen();
    LCD_Mode(SET_PIXELS);
    LCD_SetFont(BAUER_MONO_8_NORM);

    LCD_PosXY(4, 0);
    LCD_PutText("LCD graphics library");
    LCD_PosXY(28, 20);
    LCD_PutText("TEST & DEMO");

    LCD_SetFont(BAUER_PROP_8_NORM);
    LCD_PosXY(10, 40);
    LCD_PutText("Press key [1]-[6] to");
    LCD_PosXY(10, 50);
    LCD_PutText("select test screen");
}


void  DisplayTheQuickBrownFox()
{
    int   i,  x;

    LCD_ClearScreen();
    LCD_Mode(SET_PIXELS);

    LCD_PosXY(0, 22);
    LCD_DrawBar(128, 20);

    LCD_SetFont(BAUER_PROP_8_NORM);
    LCD_PosXY(16, 0);
    LCD_PutText("The quick brown fox");
    LCD_PosXY(8, 10);
    LCD_PutText("jumps over the lazy dog");
    
    LCD_SetFont(BAUER_MONO_16_NORM);
    LCD_Mode(CLEAR_PIXELS);
    LCD_PosXY(7, 24);
    LCD_PutText("Hey world!");
    LCD_Mode(SET_PIXELS);

    for (x = 0, i = 0;  i < 5;  i++, x += 27)
    {
        LCD_PosXY(x, 44);
        LCD_PutImage(test_image, 20, 20);
    }
}


void  TestImagesScreenOne()
{
    LCD_ClearScreen();
    LCD_Mode(SET_PIXELS);

    LCD_PosXY(0, 0);
    LCD_PutImage((bitmap_t *)big_right_arrow, 21, 18);

    LCD_PosXY(2, 22);
    LCD_PutImage((bitmap_t *)coffee_cup_icon, 39, 41);

    LCD_PosXY(48, 0);
    LCD_DrawLineVert(64);

    LCD_PosXY(54, 10);
    LCD_PutImage((bitmap_t *)treble_clef_16x40, 16, 40);

    LCD_PosXY(80, 0);
    LCD_PutImage((bitmap_t *)chess_knight, 44, 44);

    LCD_SetFont(BAUER_MONO_8_NORM);
    LCD_PosXY(80, 54);
    LCD_PutText("MJB 2016");
}


void  TestImagesScreenTwo()
{
    LCD_ClearScreen();
    LCD_Mode(SET_PIXELS);

    LCD_PosXY(0, 14);
    LCD_DrawBar(60, 50);

    LCD_Mode(CLEAR_PIXELS);
    LCD_PosXY(2, 16);
    LCD_PutImage((bitmap_t *)car_front_56x46, 56, 46);

    LCD_Mode(SET_PIXELS);
    LCD_PosXY(62, 2);
    LCD_PutImage((bitmap_t *)tweety_bird_63x54, 63, 54);

    LCD_SetFont(BAUER_MONO_8_NORM);
    LCD_Mode(FLIP_PIXELS);
    LCD_PosXY(2, 4);
    LCD_PutText("Mix images");
    LCD_PosXY(80, 24);
    LCD_PutText("and");
    LCD_PosXY(83, 34);
    LCD_PutText("text");

/*
    LCD_PosXY(44, 4);
    LCD_PutImage((bitmap_t *)padlock_icon, 23, 33);
*/
}


int main(void)
{
    InitializeMCUclock();
    Init_Application();

    putstr("\n");
    Cmnd_ver(1, NULL);
    PrepareForNewCommand();

    LCD_Init();
    WaitMilliseconds(200);

    LCD_ClearScreen();
    LCD_Mode(SET_PIXELS);
    LCD_BACKLIGHT_HIGH();
    DisplayHomeMessageScreen();

    while (1)
    {
        BackgroundTaskExec();
        ConsoleCLI_Service();

        if (KeyHit())
        {
            if (GetKey() == '1') DisplayTheQuickBrownFox();
            else if (GetKey() == '2') LCD_TestCharGen(BAUER_MONO_8_NORM);
            else if (GetKey() == '3') LCD_TestCharGen(BAUER_PROP_8_NORM);
            else if (GetKey() == '4') TestImagesScreenOne();
            else if (GetKey() == '5') TestImagesScreenTwo();
            else  DisplayHomeMessageScreen();
        }
    }
}


/**
 * Function:     SystemHeartbeatLEDtask()
 *
 * Overview:     Pulses the "system heartbeat" LED at 2Hz (100ms duty).
 *               Periodic B/G task called at 50ms intervals.
 */
void  SystemHeartbeatLEDtask(void)
{
    static short LED_cycle = 0;  // unit = 50ms

    if (LED_cycle >= 10) LED_cycle = 0;
    if (LED_cycle == 0) DebugLEDControl(1);
    if (LED_cycle == 2) DebugLEDControl(0);
    LED_cycle++;
}


/**
 * Background task executive...  Runs periodic tasks scheduled by the RTI timer ISR
 * and any asynchronous ("ad-hoc", non-periodic, event-driven) tasks.
 *
 * This routine is called frequently from the main loop and from inside all wait loops;
 * e.g.  while waiting for a timer to expire (see 'WaitMilliseconds()').
 * Foreground (interrupting) tasks are executed directly from their respective ISR's.
 */
void  BackgroundTaskExec(void)
{
    // Asynchronous ("ad-hoc", non-periodic) tasks may be called here...
    //

    if (v_RTI_flag_5ms_task)
    {
        // Function(s) to be executed every 5 mSec are called here...
        //
        v_RTI_flag_5ms_task = 0;
    }
    if (v_RTI_flag_50ms_task)
    {
        // Function(s) to be executed every 50 mSec are called here...
        SystemHeartbeatLEDtask();
        //
        v_RTI_flag_50ms_task = 0;
    }
    if (v_RTI_flag_500ms_task)
    {
        // Function(s) to be executed every 500 mSec are called here...
#ifdef USE_MDD_FILE_SYSTEM
        MDDmediaMountStatusUpdate();
        UpdateFSrealTimeVars();
#endif
        //
        v_RTI_flag_500ms_task = 0;
    }
    if (v_RTI_flag_NewDayRollover)
    {
        RTC_NewDayRolloverTask();
        v_RTI_flag_NewDayRollover = 0;
    }
}


/**
 * Foreground task executive...
 * Runs periodic interrupting tasks sync'd to the RTI timer.
 * The function is called by the RTI Timer ISR on every RTC "tick" (typ. 1ms).
 */
void  ForegroundTaskExec(void)
{
//  static  int   LCD_backlight_cycle = 0;

#ifdef USE_4_WIRE_KEYPAD
    KeypadService();
#endif
/*
    // Generate LCD backlight PWM drive signal at 200Hz with 1/5 duty...
    if (++LCD_backlight_cycle >= 5)
    {
        LCD_backlight_cycle = 0;
        LCD_BACKLIGHT_HIGH();
    }
    else  LCD_BACKLIGHT_LOW();
*/    
}

//########################################################################################

/*
 * CLI command function:  Cmnd_fontmod
 *
 * The command "fontmod" is intended to modify an existing font table for use with
 * the LCD graphics library. The modified font table is dumped as a C source code
 * array definition.
 * 
 */
void  Cmnd_fontmod(int argCount, char * argValue[])
{
    static  uint8  FontBuffer[100 * 32];  // enough for 96 chars x 32 bytes!

    //----- Edit the initializers below to suit the output font -------------
    char   *fontName = "font_table_prop_5x8";
    int     fontTableSize = 96 * 8;    // bytes
    int     rowsInTable = 96;
    int     bytesPerChar = 8;
    //-----------------------------------------------------------------------
    char    c = 0x20;  // Space
    uint8   bDat, width, dots;
    uint8  *pDataIn, *pDataOut;
    int     i, column = 0, row = 0;

    pDataIn = (uint8 *) font_table_mono_5x8;
    pDataOut = FontBuffer;
    rowsInTable = 96;

    putstr("const  uint8  ");  putstr(fontName);  putstr("[] = \n{ ");

#if 0  //----- Output the source array verbatim, except for end-of-line annotation. -----
       //----- All data bytes shifted 2 dots to the left.

    for (i = 0;  i < fontTableSize;  i++)
    {
        if ((column % bytesPerChar) == 0)  // start of line
        {
            putstr("\n    ");
        }

        bDat = *pDataIn << 2;  // fetch 1 byte of dots from source table
//      *pDataOut = bDat;  // store modified data byte in output table

        putstr("0x");  putHexByte(bDat);
        if (i < (fontTableSize - 1)) putstr(", ");

        if ((column % bytesPerChar) == (bytesPerChar - 1))  // end of line
        {
            putstr("  // ");
            if (c >= 0x20 && c != 0x5C && c != 0x7F) putch(c);  // not back-slash or DEL
            c++;
        }

        if (++column >= 16)  column = 0;  // new line
        pDataIn++;
        pDataOut++;
    }

#endif
#if 1  // ----- Convert 5x8 monospaced font to 5x8 proportional -------------------

    for (i = 0, row = 0;  row < rowsInTable;  row++, c++)
    {
        // First byte on the line is symbol width (bits 3..0) + descender flag (bit 7)
        width = 5;  // Default width is 5 dots;  exceptions follow...

        if (c >= 'a' && c <= 'z') width = 4;  // except...
        if (c == 'm' || c == 'v' || c == 'w' || c == 'x' || c == 'z') width = 5;

        if (c == '!' || c == '.' || c == ',' || c == ';' || c == ':' || c == 0x27
        ||  c == 'I' || c == 'i' || c == 'j' || c == 'l' ) width = 3;
        
        bDat = width;

        if (c == ',' || c == ';' || c == '_' || c == 'g' || c == 'j' || c == 'p'
        ||  c == 'q' || c == 'y') bDat |= 0x80;  // add descender flag

        if (c >= '0' && c <= '9')
        {
            putstr("\n    0x");  putHexByte(bDat);  putstr(", ");  // output width byte
        }

        for (column = 0;  column < (bytesPerChar - 1);  column++)
        {
            bDat = *pDataIn++;   // fetch 1 byte from source table
            dots = (bDat & 0x18) << 1;  // RH 2 dots

            if (c >= '0' && c <= '9')
            {
                bDat = (bDat & 0xC0) | dots;   // 2 LH dots + 2 RH dots
                putstr("0x");  putHexByte(bDat);  putstr(", ");
            }

//          if (i < (fontTableSize - 1)) putstr(", ");
            i++;
        }

        if (c >= '0' && c <= '9')
        {
            putstr("  // ");   // end of line
            if (c >= 0x20 && c != 0x5C && c != 0x7F)  putch(c);  // not back-slash or DEL
        }
        
        pDataIn++;  // skip 1 byte (bottom row of dots)
    }

#endif
    
    putstr("\n}; \n\n");
}


/**
*   Function called by "diag" command function...
*/
void  DiagnosticCommandExec(int argCount, char * argValue[])
{
    char   cBuf[80];
    char   option = tolower(argValue[1][1]);

//  if (!SuperUserAccess()) return;

    if ( argCount == 1 || *argValue[1] == '?' )   // help wanted
    {
        putstr( "System low-level diagnostics \n" );
        putstr( "Usage:  diag  <option>  [arg] ... \n" );
        putstr( "<option> \n" );
        putstr( " -d :  Display test pattern on LCD panel \n");
        putstr( " -k :  Keypad test \n");
        ////
        return;
    }

    switch (option)
    {

        case 'd':  // Display LCD test pattern
        {
            char  c = 0;

            DisplayTheQuickBrownFox();

            putstr("<!> B/G tasks are suspended... \n");
            putstr("Press [Esc] to exit LCD test. \n");

            while (c != ASCII_ESC)
            {
                Delay_ms(50);
                if (kbhit()) c = getch();
            }

            DisplayHomeMessageScreen();
            break;
        }
        case 'k':  // Keypad test
        {
            char  c = 0;

            putstr("<!> B/G tasks are suspended... \n");
            putstr("Press a key on the keypad to test it. \n");
            putstr("Press [Esc] on terminal to exit test. \n\n");

            while (c != ASCII_ESC)
            {
                if (KeyHit())
                {
                    putch(GetKey());
                }

                if (kbhit()) c = getch();
            }

            putstr("\n");
            break;
        }
        default:  break;

    } // end switch
}


/**
*   Function called by "watch" command function...
*   Variables to be "watched" are output on a single line (no newline).
*/
void  WatchCommandExec(void)
{
    char   textBuf[100];
    short  adc_keyX, adc_keyY;
    int    keyDownTime;

    adc_keyX = GetADCreadingKeyCol();
    adc_keyY = GetADCreadingKeyRow();
    keyDownTime = GetKeyPressTime_ms();

    sprintf(textBuf, "Keypad ADC X = %5d, Y = %5d, key down %5d ms ",
            adc_keyX, adc_keyY, keyDownTime);
    putstr(textBuf);
}


/**
*   Function called by "default" command function...
*/
void  DefaultPersistentData(void)
{
    ;;;
}


/**
*   Function called by "commit" command function...
*/
void  CommitPersistentParams(void)
{
    ;;;
}


/**
*   Function called by "eeprom" command...
*/
void    Cmnd_eeprom(int argCount, char * argValue[])
{
    putstr("! Not implemented: eeprom \n");
}

